﻿unit EccHtml;

interface

uses
  EccDefs;

var
  IndexProc: IndexProcRef;

function GenerateHtml: string;

implementation

uses SysUtils, StrUtils, System.Classes,
  EccProcs, EccMemos, EccCSlav, EccLists, EccPrays, EccVars;

resourcestring
  HtmlHead1 = '<!DOCTYPE html">' +
    '<html lang="ru"><head><meta http-equiv="Content-Type" content="text/html; charset=Windows-1251"/>' +
    '<meta name="Generator" content="Ecclysiarch 1.0"/><title></title>' +
    '<style type="text/css">' +
    '  * {margin: 0; padding: 0; border: 0; font-family: "Orthodox.tt ieUcs8"; font-weight: normal; font-style: normal; line-height: 110%}' +
    '  body {margin: 10px}' +
    '  body, p, td, h4 {color: black; font-size: 32px}' +
    '  h1, h3, h4, h5, .tit {text-align: center}' +
    '  .tit {font-family: "Kathisma ieUcs"; font-size: 200%; line-height: 120%}' +
    '  p, td {text-align: justify; text-indent: 24px; padding-bottom: 4px}' +
    '  h6 {text-indent: 60px; padding-top: 20px}' +
    '  table {width: 100%; margin-bottom: -2px; border-collapse: collapse}' +
    '  td {vertical-align: top; padding-right: 20px}' +
    '  td+td {padding-left: 20px; border-left: 1px solid silver}' +
    '  td:last-child {padding-right: 0px}' +
    '  h5, h6, em, .spoiler {font-size: 85%}' +
    '  b, h1, h2, h3, h4, h5, h6, em, .tit {color: red}' +
    '  h1, h2 {display: none}' +
    '  .spoiler {font-family: Arial; color: silver; cursor: pointer}' +
    '  h3, h4, h5 {line-height: 100%; padding-top: 10px; padding-bottom: 10px}' +
    '  h3 {font-size: 120%}' +
    '  .s {color: black}' +
    '  .tip {position: fixed; padding: 5px 20px 10px; border: 1px solid #b3c9ce; ' +
    'text-align: center; font-size: 90%; color: black;' +
    'background: rgb(222, 222, 222)}' +
    '  .info {color: rgb(0, 111, 111)}' +
    '  .noselect {-webkit-touch-callout: none; -webkit-user-select: none; -khtml-user-select: none; ' +
    '-moz-user-select: none; -ms-user-select: none; user-select: none}' +
    '</style>' +

    '<script lang="javascript">' +

    '  var showingTooltip;' +
    '  function showtip(target) {' +
    '    var tooltip = target.getAttribute("tip");' +
    '    if (!tooltip) return; ' +
    '    var tooltipElem = document.createElement("div");' +
    '    tooltipElem.className = "tip";' +
    '    tooltipElem.innerHTML = tooltip;' +
    '    document.body.appendChild(tooltipElem);' +
    '    var coords = target.getBoundingClientRect();' +
    '    var left = coords.left + (target.offsetWidth - tooltipElem.offsetWidth) / 2;' +
    '    if (left < 0) {left = 0};' +
    '    var top = coords.top - tooltipElem.offsetHeight - 5;' +
    '    if (top < 0) {top = coords.top + target.offsetHeight + 5 }' +
    '    tooltipElem.style.left = left + "px";' +
    '    tooltipElem.style.top = top + "px";' +
    '    showingTooltip = tooltipElem };' +

    '  function hidetip() {' +
    '    if (showingTooltip) {' +
    '      document.body.removeChild(showingTooltip); ' +
    '      showingTooltip=null }};' +

    '  function sw(n) {' +
    '    obj=n.parentNode.parentNode.childNodes[1].style;' +
    '    tmp=(obj.display!="block") ? "block" : "none";' +
    '    obj.display=tmp;' +
    '    if (tmp=="block") {n.innerHTML="\u25B2"}' +
    '    else {n.innerHTML="\u25BC"};' +
    '    return false };';
  HtmlHead2 =
    '  var threshold = 5;' +
    '  var resizingTdLeft;' +
    '  var resizingTdRight;' +
    '  var resizingTdLeftStartingWidth;' +
    '  var resizingTdRightStartingWidth;' +
    '  var startX;' +
    '  var paddingWidth = 20;' +
    '  var borderWidth = 1;' +

    '  function getPreviousSibling(element) {' +
    '    var x = element.previousSibling;' +
    '    while (x && x.nodeType != 1 && x.tagName != "TD") {' +
    '      x = x.previousSibling };' +
    '    return x };' +

    '  function getNextSibling(element) {' +
    '    var x = element.nextSibling;' +
    '    while (x && x.nodeType != 1 && x.tagName != "TD") {' +
    '      x = x.nextSibling };' +
    '    return x };' +

    '  function onMouseDown(event) {' +
    '    event = event || window.event;' +
    '    var target = event.target || event.srcElement;' +
    '    var coords = target.getBoundingClientRect();' +
    '    var previousTd;' +
    '    var nextTd;' +
    '    var previousPreviousTd;' +
    '    var nextNextTd;' +
    '    if (isLeftResizeAvailable(event)) {' +
    '      resizingTdLeft = getPreviousSibling(target);' +
    '      resizingTdRight = target }' +
    '    else if (isRightResizeAvailable(event)) {' +
    '      resizingTdLeft = target;' +
    '      resizingTdRight = getNextSibling(target) };' +
    '    if (resizingTdLeft && resizingTdRight) {' +
    '      resizingTdLeftStartingWidth = resizingTdLeft.offsetWidth - (paddingWidth + borderWidth);' +
    '      resizingTdRightStartingWidth = resizingTdRight.offsetWidth - (paddingWidth + borderWidth);' +
    '      resizingTdLeft.style.cursor = "col-resize";' +
    '      resizingTdRight.style.cursor = "col-resize";' +
    '      resizingTdLeft.className = "noselect";' +
    '      resizingTdRight.className = "noselect";' +
    '      document.onselectstart = function() { return false; };' +
    '      startX = event.clientX }};' +

    '  function isLeftResizeAvailable(event) {' +
    '    var target = event.target || event.srcElement;' +
    '    if (target.tagName != "TD") return false;' +
    '    var coords = target.getBoundingClientRect();' +
    '    return event.clientX - coords.left < threshold && getPreviousSibling(target) !== null };' +

    '  function isRightResizeAvailable() {' +
    '    var target = event.target || event.srcElement;' +
    '    if (target.tagName != "TD") return false;' +
    '    var coords = target.getBoundingClientRect();' +
    '    return event.clientX - coords.left - target.clientWidth > -threshold && getNextSibling(target) !== null }' +

    '  function onMouseMove(event) {' +
    '    event = event || window.event;' +
    '    if (resizingTdLeft && resizingTdRight) {' +
    '      resizingTdLeft.style.width = resizingTdLeftStartingWidth + (event.clientX - startX) + "px";' +
    '      resizingTdRight.style.width = resizingTdRightStartingWidth - (event.clientX - startX) + "px" }' +
    '    else {' +
    '      if (isLeftResizeAvailable(event) || isRightResizeAvailable(event)) {' +
    '        document.body.style.cursor = "col-resize" }' +
    '      else {' +
    '        document.body.style.cursor = "auto" }}};' +

    '  function stopResizing() {' +
    '    document.onselectstart = null;' +
    '    if (resizingTdLeft) {' +
    '      resizingTdLeft.style.cursor = "";' +
    '      resizingTdLeft.className = "";' +
    '      resizingTdLeft = null };' +
    '    if (resizingTdRight) {' +
    '      resizingTdRight.style.cursor = "";' +
    '      resizingTdRight.className = "";' +
    '      resizingTdRight = null }};' +

    '  window.onload = function() {' +
    '    var tr=document.getElementsByTagName("tr");' +
    '    for (j=tr.length-1; j>=0; j--) {' +
    '      tr[j].lastChild.style.paddingRight=0;' +
    '      c=tr[j].childNodes.length;' +
    '      if (c==1) {continue};' +
    '      sum=0;' +
    '      for (i=0; i<tr[j].childNodes.length; i++) {' +
    '        sum=sum+tr[j].childNodes[i].childNodes[0].offsetHeight*tr[j].childNodes[i].childNodes[0].offsetWidth };' +
    '      for (i=0; i<tr[j].childNodes.length; i++) {' +
    '        var w=tr[j].childNodes[i].childNodes[0].offsetHeight*tr[j].childNodes[i].childNodes[0].offsetWidth/sum*100+"%";' +
    '        tr[j].childNodes[i].style.width=w }};' +
    '    var tds = document.getElementsByTagName("td");' +
    '    for (var i = 0; i < tds.length; i++) {' +
    '      tds[i].onmousedown = onMouseDown }};' +

    '  if (typeof document.attachEvent != "undefined") {' +
    '    document.attachEvent("onmousemove", onMouseMove);' +
    '    document.attachEvent("onmouseup", stopResizing) }' +
    '  else {' +
    '    window.onmousemove = onMouseMove;' +
    '    window.onmouseup = stopResizing }' +
    '</script></head><body>';
    HtmlFooter = '</body></html>';

var
  LastStyle, LastType, LastVoice, LastSubv, AncorID: Integer;
  LastRoles: TRolesList;

function InsertShysToWord(const S: string): string;
var
  i, l: Integer;
const
  _shy: string = '&shy;';
begin
  l := s.Length;
  if l < 4 then Exit(S);
  if CharInString(s.Chars[l - 1], _PuncSet + '/') then
    Exit(InsertShysToWord(s.Substring(0, l - 1)) + s.Chars[l - 1]);
  Result := s.Chars[0];
  i := Result.Length - 1;
  if l - i < 4 then Exit(S);
  while (i < l - 4) or ((i = l - 4) and not CharInString(s.Chars[l - 1], _UdarSet)) do
    begin
      Inc(i);
      if CharInString(s.Chars[i], _TitlSet + _TitbSet) then
        begin
          Result := Result + s.Substring(i, 2);
          Inc(i);
          Continue;
        end;
      if CharInString(s.Chars[i - 1], _TitlSet + _TitbSet) then
        begin
          Result := Result + s.Chars[i];
          Continue;
        end;
      if CharInString(s.Chars[i], _SoglSet) then
        if (CharInString(s.Chars[i - 1], _GlasSet + _UdarSet) or ((i > 1) and CharInString(s.Chars[i - 2], _GlasSet + _UdarSet))) and CharInString(s.Chars[i + 1], _SoglSet) and CharInString(s.Chars[i + 2], _GlasSet) then
          begin
            Result := Result + s.Chars[i] + _shy + s.Chars[i + 1];
            Inc(i);
            Continue;
          end
        else
          begin
            Result := Result + s.Chars[i];
            Continue;
          end;
      if CharInString(s.Chars[i], _SignSet) then
        begin
          Result := Result + s.Chars[i] + _shy + s.Chars[i + 1];
          Inc(i);
          Continue;
        end;
      if (CharInString(s.Chars[i], _GlasSet) or (CharInString(s.Chars[i], _UdarSet) and (i > 1))) and
        (CharInString(s.Chars[i + 1], _GlasSet + _TitbSet) or CharInString(s.Chars[i + 2], _GlasSet + _TitlSet)) then
        Result := Result + s.Chars[i] + _shy
      else Result := Result + s.Chars[i];
    end;
  Result := Result + s.Substring(i + 1);
end;

function InsertShys(S: string): string;
var
  d, m: string;
  j: Boolean;

  function _insert(s: string): string;
    var
      d: string;
    begin
      Result := '';
      repeat
        j := StrDivide(s, d, s, #32);
        Result := Result + InsertShysToWord(d);
        if j then Result := Result + #32;
      until s.Trim.IsEmpty;
    end;

begin
  if not _s_Perenos then Exit(S);
  Result := '';
  repeat
    StrDivide(S, d, S, '<');
    StrDivide(S, m, S, '>');
    if not d.IsEmpty then
      Result := Result + _insert(d);
    if not m.IsEmpty then
      Result := Result + '<' + m + '>';
  until S.IsEmpty;
end;

function FirstCapTag(const S: string): string; {opt}
var
  i: Integer;
  d, m, ss: string;
  l: Boolean;
begin
  Result := '';
  ss := S;
  repeat
    StrDivide(ss, d, ss, '<');
    l := d.Trim.IsEmpty and not ss.IsEmpty;
    if l then
      begin
        StrDivide(ss, m, ss, '>');
        Result := Result + d;
        if not m.IsEmpty then Result := Result + '<' + m + '>';
      end;
  until not l;
  i := FirstLength(d);
  if i = 0 then Exit(S);
  Result := Result + FirstCap(d);
  if not ss.IsEmpty then Result := Result + '<' + ss;
end;

function FirstBold(const S: string; Need: Boolean): string; {opt}
var
  i: Integer;
  sl: TStringList;

  function _do(const s: string): string;
    var
      i: Integer;
      d, f, m, ss: string;
      l: Boolean;
    begin
      Result := '';
      ss := S;
      repeat
        repeat
          StrDivide(ss, d, f, '<em>');
          l := d.Trim.IsEmpty and not f.IsEmpty;
          if l then      // есть <em>, и он вначале
            begin
              StrDivide(f, m, ss, '</em>');
              Result := Result + d;
              if not m.IsEmpty then
                Result := Result + '<em>' + m + '</em>';
            end;
        until not l;
        StrDivide(d, d, ss, '<');
        l := d.Trim.IsEmpty and not ss.IsEmpty;
        if l then
          begin
            StrDivide(ss, m, ss, '>');
            Result := Result + d + '<' + m + '>';
          end;
      until not l;
      i := FirstLength(d);
      if i = 0 then Exit(S);
      Result := Result + '<b>' + d.Substring(0, i) + '</b>' + d.Substring(i) ;
      if not ss.IsEmpty then Result := Result + '<' + ss;
      if not f.IsEmpty then Result := Result + '<em>' + f;
    end;

begin
  if not Need then Exit(S);
  Result := '';
  sl := TStringList.Create;
  AddStringList(sl, S.Replace(#9, #10));
  for i := 0 to sl.Count - 1 do
    Result := Result + #32 + _do(sl[i]);
  sl.Free;
  Result := Trim(Result);
end;

function ProcessBoldInfo(S: string): string;
var
  s1, s2, s3: string;
  i: Integer;
begin
  Result := S;
  while DispatchSubtag('B', Result, s1, s2, s) do
    if _s_FirstLetter then
      begin
        i := FirstLength(s2);
        Result := s1 + '<b>' + s2.Substring(0, i) + '</b><b class="s">' + s2.Substring(i) + '</b>' + s;
      end
    else
      Result := s1 + s2 + s;
  while DispatchSubtag('Z', Result, s1, s2, s) do
    begin
      StrDivide(s2, s2, s3, '[');
      if BoolVar(SysVars[21]) then
        begin
          s3 := s3.Remove(s3.Length - 1);
          if BoolVar(SysVars[22]) then
            Result := s1 + '<span tip="' + s3 + '" onmouseover="showtip(this);" onmouseout="hidetip();">°' + s2 + '°</span>' + s
          else
            Result := s1 + '<span class="info" tip="' + s3 + '" onmouseover="showtip(this);" onmouseout="hidetip();">' + s2 + '</span>' + s;
        end
      else Result := s1 + s2 + s;
    end;
end;

procedure ResetAllLasts;
begin
  SetLength(LastRoles, 0);
  LastStyle := -1;
  LastType := -1;
  LastVoice := -1;
  LastSubv := -1;
end;

function IsRolesEqu(const A, B: TRolesList): Boolean;
begin
  Result := Length(A) = Length(B);
  if not Result then Exit;
  Result := True;
  for var i := Low(A) to High(A) do
    if (A[i].Role <> B[i].Role) or (A[i].Subr <> B[i].Subr)
      or (A[i].Count <> B[i].Count) then Exit(False);
end;

function ProcessPray(const P: TPrayRec): string; forward;

function GeneratePretag(Pray: TPrayRec): string;
var
  s, n, m: string;
  v, st: Boolean;
begin
  if _s_Notbreaks then n := '&nbsp;' else n := #32;
  Result := '';
  case Pray.Mode of
    0: begin
      if Length(Pray.Roles) = 0 then Pray.Roles := LastRoles;
      if not IsRolesEqu(Pray.Roles, LastRoles) then Result := Pray.RolesStr;
      if Pray.Style = -1 then Pray.Style := LastStyle;
      if (Length(Pray.Roles) > 0) and (((Pray.Style <> LastStyle) and (Pray.Style <> 0) and not ((Pray.Roles[0].Role = 2) and (Pray.Style = 1))
        and not ((Pray.Roles[0].Role = 5) and (Pray.Style = 2))) or
        ((Pray.Style = 0) and ((LastStyle <> 0) and IsRolesEqu(Pray.Roles, LastRoles))) or
        ((Pray.Style = 1) and not (Pray.Roles[0].Role = 2) and not IsRolesEqu(Pray.Roles, LastRoles))) then
        begin
          if Result <> '' then Result := Result + ',' + n
          else Result := RolesList[0].Name[1] + n;
          Result := Result + ApplyPrayGramm(StylesList[Pray.Style], Pray.RolesCount, Pray.Iter, GloryDic[Pray.GloryId].Count);
        end;
      LastStyle := Pray.Style;
      LastRoles := Pray.Roles;
      if Result <> '' then Result := FirstCap(Result + ':' + n);
      s := '';
      v := False;
      if (Pray.Title <> '') and (Pray.Title[1] <> '#') then
        begin
          s := InsertShys(ProcessBoldInfo(Pray.Title));
          v := True;
        end
      else
        if (Pray.PrayType <> LastType) and (IndexStr(Pray.PrayType.ToString, _sTypesWrite) >= 0)
          and (Pray.Id <> 'ch.comm_14') then
          begin
            s := TypesList[Pray.PrayType];
            v := True;
          end;
      st := IndexStr(Pray.PrayType.ToString, _sStih) >= 0;
      if not st then LastType := Pray.PrayType;
      if (Pray.Voice <> LastVoice) and (Pray.Voice in [1..8])
      and (IndexStr(LastStyle.ToString, _sNoVoiceAfterStyle) < 0) then
        begin
          v := True;
          if st then s := ''; {чтобы не было написания "Стих, глас 5"}
          if s <> '' then s := s + ',' + n;
          s := s + VoicesList[0] + '&nbsp;' + CSNum(Pray.Voice);
        end;
      m := '';
      if (Pray.SubVoice <> LastSubv)
      and (Pray.SubVoice <= High(SubVoicesList))
      and (Pray.SubVoice > 0) and (LastStyle = 1)
      and (Pray.Voice in [1..8]) and not st then
        begin
          if v then s := s + ',' + n;
          if (Pray.SubVoice > 2) and _s_FirstLetter then
            begin
              m := SubVoicesList[Pray.SubVoice];
              Insert('<b class="s">', m, FirstLength(m) + 1);
              s := s + SubVoicesList[0] + ':' + n + m;
              m := '</b>';
            end
          else
            s := s + SubVoicesList[Pray.SubVoice];
        end;
      if not st then LastSubv := Pray.SubVoice;
      LastVoice := Pray.Voice;
      if s <> '' then s := FirstCap(s + ':' + n + m);
      Result := Result + s;
      if (Pray.Subscr <> '') and BoolVar(SysVars[23]) then Result := ProcessBoldInfo(Pray.Subscr) + '</em></p><p><em>' + Result;
      if Result <> '' then Result := '<em>' + Result + '</em>';
    end;
    1: begin
      if not IsRolesEqu(Pray.Roles, LastRoles) and (Pray.RolesCount > 0) then Result := Pray.RolesStr
      else if Pray.RolesCount > 0 then Result := 'И#';
      LastRoles := Pray.Roles;
      LastStyle := -1;
    end;
  end;
end;

function InsertActs(S: string; Rl: TRolesList): string;
var
  s1, s2, s3, s4, m: string;
  p: TPrayRec;
  n: Integer;
  r1: TRolesList;

  procedure _strDivide(const s: string; var s1, s2: string; c: WideChar);
  var
    i, k: Integer;
  begin
    k := 0;
    for i := 0 to s.Length - 1 do
      begin
        if (s.Chars[i] = c) and (k = 0) then
          begin
            s1 := s.Substring(0, i);
            s2 := s.Substring(i + 1);
            Exit;
          end
        else if (s.Chars[i] = ']') and (k > 0) then Dec(k)
        else if s.Chars[i] = '[' then Inc(k);
      end;
    s1 := s;
    s2 := '';
  end;

begin
  Result := S;
  while DispatchSubtag('A', Result, s1, s2, s) do
    begin
      if _s_InsertActs then
        begin
          n := 1;
          r1 := Rl;
          _strDivide(s2, s2, s3, ':');
          if not s3.IsEmpty then
            begin
              _strDivide(s3, s3, s4, ':');
              n := s3.ToInteger;
              if not s4.IsEmpty then r1 := UnpackRoles(s4);
            end;
          s2 := s2 + ',';
          m := '';
          while Pos(',', s2) > 0 do
            try
              _strDivide(s2, s3, s2, ',');
              StrDivide(s3, s4, s3, '[');
              ActDic[s4].ToRec(p);
              p.Roles := r1;
              if not IsRolesEqu(Rl, r1) then
                m := m + p.RolesStr + ': ';
              if p.Title <> '' then m := m + p.Title + ', '
              else
                m := m + ApplyObj(ApplyFlags(ApplyPrayGramm(p.Text, p.RolesCount, n, GloryDic[p.GloryId].Count)), '[L:' + s3)  + ', ';
            except
              {$IFDEF DEBUG}
              OutMessage('Несуществующее действие в службе:'#13#10 + s4);
              {$ENDIF}
            end;
          SetLength(m, Length(m) - 2);
          Result := s1 + ' <em>(' + m;
          if n > 1 then
            Result := Result + ', <b>' + CSNum(n) + '</b>';
          if CharInString(s.Chars[0], _PuncSet) then
            Result := Result + ')' + s[1] + '</em>' + s.Substring(1)
          else
            Result := Result + ')</em>' + s;
        end
      else Result := s1 + s;
    end;
end;

function HtmlEncodePray(P: TPrayRec): string;
var
  LineBreaked: Boolean;
  j, n, t: Integer;
  s: string;
begin
  s := P.PrayType.ToString;
  LineBreaked := (IndexStr(s, _sNoLineBreak) < 0) and _s_LinesMerge;
  if (IndexStr(s, _sNoPretag) < 0) then Result := GeneratePretag(P) else Result := '';
  if (IndexStr(s, _sReplaceStars) >= 0) and _s_InsertStars then
    P.Text := P.Text.Replace('/', '*')
  else P.Text := P.Text.Replace('/', '');
  with TStringList.Create do
    begin
      Text := ProcessBoldInfo(P.Text);
      for n := 0 to Count - 1 do
        begin
          s := InsertActs(Strings[n], P.Roles);
          if (Strings[n].Substring(0, 3) = '[A:') and _s_InsertActs then
            if Result <> '' then
              begin
                j := s.IndexOf('(');
                Result.Insert(Result.Length - 7, #32 + s.Substring(j, s.IndexOf(')') - j + 1));
                s := s.Substring(s.IndexOf(')') + 6);
              end
            else
              s := s.Insert(s.IndexOf(')') + 1, ': ');
          s := InsertShys(FirstBold(FirstCapTag(s), ((n = 0) or LineBreaked) and _s_FirstLetter));
          if (n = Count - 1) and ((P.Iter > 1) or (P.Iter < 0)) then
            begin
              j := Length(s) - 1;
              if s[j + 1] = ')' then
                begin
                  t := PosExR('(', s);
                  Result := Result + s.Substring(0, t - 2) + ', <b>' + CSNum(P.Iter) + s.Substring(t - 1) + s[t - 1] + '</b>';
                end
              else
                if CharInString(s.Chars[j], _PuncSet) then
                  Result := Result + s.Substring(0, j) + ', <b>' + CSNum(P.Iter) + s[j + 1] + '</b>'
                else
                  if s.Chars[j] = '>' then
                    Result := Result + s.Substring(0, j - 5) + ',</em> <b>' + CSNum(P.Iter) + '.</b>'
                  else
                    Result := Result + s + ', <b>' + CSNum(P.Iter) + '.</b>';
            end
          else Result := Result + s;
          if n < Count - 1 then
            if LineBreaked then Result := Result + '</p><p>'
            else Result := Result + #32;
        end;
      Free;
    end;
  Result := '<p>' + Result + '</p>';
  if (P.PrayType in [56]) then Result := Result.Replace('<p>','<p><em>', [rfReplaceAll]).Replace('</p>','</em></p>', [rfReplaceAll]);
end;

function HtmlEncodeAct(const P: TPrayRec): string;
begin
  if not _s_InsertActs then Exit('');
  Result := '<p><em>' + FirstCap(GeneratePretag(P)) + #32 + InsertShys(ApplyPrayGramm(P.Text, P.RolesCount, P.Iter, 1));
  if P.Iter > 1 then Result := Result + ', <b>' + CSNum(P.Iter) + '.</b>'
  else Result := Result + '.</em>';
  Result := Result + '</p>';
end;

function HtmlEncodeBlock(const P: TPrayRec): string;
var
  i, j, c: Integer;
  n, s: string;

  function _ipr(const s: string): string;
    begin
      Result := s;
      while Result.IndexOf('[') > 0 do Result := Result.Substring(0, Result.IndexOf('[')) + Result.Substring(Result.IndexOf(']') + 1);
    end;

begin
  {$IFDEF DEBUG}
  Inc(_NodeLevel);
  {$ENDIF}
  Result := '';
  c := P.Items.Count;
  if (c = 0) or (P.Items[0].Count = 0) then Exit;
  if Assigned(IndexProc) then IndexProc(_ipr(P.Text), P.Style, AncorId);
  if c > 1 then
    begin
      if _s_SyncColumns then ResetAllLasts;
      for i := 0 to c - 1 do
        begin
          if P.Items[i].Count = 0 then Continue;
          if _s_SyncColumns then Result := Result + '<td><div style="display: block">';
          for j := 0 to P.Items[i].Count - 1 do
              Result := Result + ProcessPray(P.Items[i][j]);
          if _s_SyncColumns then Result := Result + '</div></td>';
        end;
      if _s_SyncColumns and not Result.IsEmpty then
        begin
          Result := '<table><tr>' + Result + '</tr></table>';
          ResetAllLasts;
        end;
    end
  else
    begin
      n := IntToStr(P.Style);
      s := ProcessBoldInfo(P.Text).Replace('[', '', [rfReplaceAll]).Replace(']', '', [rfReplaceAll]);
      if _s_Spoilers then
        begin
          Result:= '<table><tr><td class="seq"><h' + n + '>' + s + '<a onclick="sw(this);" class="spoiler">';
          if _s_SpExpanded or (P.Style = 2) or (P.Iter = 1) then
            Result := Result + '&#x25B2;</a></h' + n + '><div style="display: block">'
          else
            Result := Result + '&#x25BC;</a></h' + n + '><div style="display: none">';
          ResetAllLasts;
        end
      else
        if _s_SpTitles or (P.Style = 2) then Result := '<h' + n + '>' + s + '</h' + n + '>';
      for j := 0 to P.Items[0].Count - 1 do
          Result := Result + ProcessPray(P.Items[0][j]);
      Result := '<a name="z' + AncorId.ToString + '"></a>' + Result;
      Inc(AncorId);
      if _s_Spoilers then
        begin
          Result := Result + '</div></td></tr></table>';
          ResetAllLasts;
        end;
    end;
  {$IFDEF DEBUG}
  Dec(_NodeLevel);
  {$ENDIF}
end;

function ProcessPray(const P: TPrayRec): string;
begin
  {$IFDEF DEBUG}
  case P.Mode of
    0: DebugWrite('+ pray ' + P.Id, True);
    1: DebugWrite('+ act ' + P.Id, True);
    2: DebugWrite('+ seq ' + P.Title, True);
    3: DebugWrite('+ title ' + P.Style.ToString, True);
    4: DebugWrite('+ img ' + P.Id, True);
  end;
  {$ENDIF}
  case P.Mode of
    0: Result := HtmlEncodePray(P);
    1: Result := HtmlEncodeAct(P);
    2: Result := HtmlEncodeBlock(P);
    3: if _s_SpTitles then
      if P.Id.IsEmpty then
        Result := '<h' + IntToStr(P.Style) + '>' + FirstCapTag(P.Text) + '</h' + IntToStr(P.Style) + '>'
      else
        Result := '<p class="' + P.Id + '">' + FirstCapTag(P.Text) + '</p>';
    4: if _s_InsertImgs then Result := '<h5><img src="' + P.Text + '"/></h5>';
  end;
end;

function GenerateHtml: string;
var
  sl: TStringList;
  i: Integer;
begin
  sl := TStringList.Create;
  AncorId := 0;
  ResetAllLasts;
  _NodeLevel := 0;
  try
    sl.Add(HtmlHead1 + HtmlHead2);
    for i := 0 to PrayList.Count - 1 do
      sl.Add(ProcessPray(PrayList[i]));
    sl.Add(HtmlFooter);
  finally
    {$IFDEF DEBUG}
    sl.SaveToFile('temp.htm');
    {$ENDIF}
    Result := sl.Text;
    sl.Free;
  end;
end;

end.


